import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.text.*;
/**
 * La classe Balise reprensente une balise Xml <br>
 * Il s'agit ici de la description du type de la balise (et non de la valeur reel de cette balise
 * dans le document Xml)<br>
 * Cette description implique et sa description dans la dtd et sa representation en terme d'IHM
 *  @author Gros Pierre Emmanuel
 *         <a href="mailto:gros@limsi.fr">gros@limsi.fr</a>
 * @version 6.0
 */
public class Balise implements Serializable
{
/**
 * Le nom de la balise
 */
protected String _name;
/** 
 * La balise est t'elle du type empty
 */
protected boolean _empty;
/**
 * Attribut donnant la representation graphique de la balise
 */
protected SimpleAttributeSet _IhmAttribut;
protected boolean _default;
/**
 * La liste des attributs
 */
 protected SemantiqueRule _rule;
protected Vector _liste_attribut;
int _order;
/**
 * Construit la balise <br>
 * @param     name est le nom de la balise <br>
 * @param     empty doit etre a vrai si la balise est du type vide
 */
public Balise(String name,boolean empty)
{
if (name!=null) _name=new String(name);
_empty=empty;
_IhmAttribut=new SimpleAttributeSet();
StyleConstants.setForeground(_IhmAttribut, Color.black);
StyleConstants.setBackground(_IhmAttribut,Color.white);
_liste_attribut=new Vector(10,10);
_default=true;
_rule=new SemantiqueRule();
}
/**
 * setOder met a jour l'ordre de la balise
 * @param order l'ordre de la balise
 */
 public SemantiqueRule getRule(){return _rule;}
 public void setRule(SemantiqueRule rule){_rule=rule;}
public void setOrder(int order)
{
_order=order;
}
/**
 * renvoie l'ordre de la balise
 * @result l'ordre de la balise
 */
public int getOrder()
{
return(_order);
}
/**
 * renvoie vrai si la balise est du type vide 
 * @result vrai si la balise est vide
 */
public boolean IsEmptyBalise()
{
return _empty;
}


private void writeObject(java.io.ObjectOutputStream out)
     throws IOException
     {
  
     	out.writeUTF(_name);
     	out.writeBoolean(_empty);
     	out.writeObject(_IhmAttribut);
     	out.writeObject(_liste_attribut);
     	out.writeInt(_order);
     }
private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException
     {
     	_name = in.readUTF();
	_empty = in.readBoolean();
	_IhmAttribut=(SimpleAttributeSet) in.readObject();
	_liste_attribut=(Vector) in.readObject();
	_order=in.readInt();

}

/**
 * Renvoie la representation graphique de la balise
 * @result le style graphique de la balise
 */
public SimpleAttributeSet getAttribut(){return _IhmAttribut;}
/**
 * Renvoie le nom de la balise
 * @result le nom de la balise
 */
public String getName(){return _name;}
/** 
 * Renvoie une chaine de caractere representant la balise
 * @result une chaine de caractere 
 */
public String toString(){if (!_empty) {return new String (_name);}
		else {return new String(_name+new String("[Empty]"));}}
/**
 * Renvoie la liste des attributs de la balise
 * @result un vecteur d'objet AttributDecl
 * @see       AttributDecl()
 *     
 */
public Vector getListeAttribut(){return _liste_attribut;}
/**
 * Add ajoute une declaration d'attribut a la liste des attribut de la balise
 * @param a la declaration d'un attribut
 */
public void Add(AttributDecl a)
{
_liste_attribut.add(a);
}

/**
 * met a jour le style graphique de la balise
 * @param attr le style graphique
 * @param ecrase vrai si on veut remplacer le style graphique , faux si on veut le fusionner
 */
public void UpDateAttributDefault(AttributeSet attr,boolean ecrase)
{

if (_default || ecrase) {_IhmAttribut.addAttributes(attr);_default=true;}

}
/**
 * Mets a jour les attributs graphiques de la balise
 * @param attr le style graphique de la balise
 *
 */
public void UpDateAttribut(AttributeSet attr)
{


_IhmAttribut.addAttributes(attr);

}
}